require 'uri'
require 'faraday'

module WCC::Data
  class Config
    def initialize
      set_default_applications
    end

    def applications
      @applications ||= Hash.new { |hash, key| hash[key] = Application.new(key) }
    end
    alias :apps :applications

    private

    def set_default_applications
      raise ArgumentError, "Missing NUCLEUS_URL environment variable" unless ENV['NUCLEUS_URL']
      applications[:nucleus].uri = ENV['NUCLEUS_URL']
      if ENV['APP_CLIENT_ID'] && ENV['APP_CLIENT_SECRET']
        applications[:nucleus].connection
          .basic_auth(ENV['APP_CLIENT_ID'], ENV['APP_CLIENT_SECRET'])
      end
    end

    class Application
      attr_reader :name, :uri
      attr_accessor :connection

      def initialize(name, connection: Faraday::Connection.new)
        @name = name
        @connection = connection
      end

      def uri=(uri)
        @uri = URI(uri)
      end

      def service
        Service.new(connection: connection, uri: uri)
      end

    end

  end
end
