module WCC::Data
  class EnumeratedType

    def initialize(args={})
      only_attributes(args).each do |key, value|
        instance_variable_set("@#{key}", value)
      end
    end

    def self.[](value)
      all.find { |record| record.matches?(value) }
    end

    def matches?(value)
      raise NotImplementedError,
        "The #matches? method should be defined in subclasses and return " \
        "true if the argument matches this record and false otherwise."
    end

    def self.db
      raise NotImplementedError,
        "The ::db class method should be defined in subclasses as an array " \
        "of hashes containing data for each record."
    end

    def self.all
      @all ||= db.collect { |data| new(data) }
    end

    def self.reset
      @all = nil
    end

    private

    def only_attributes(args)
      args.select { |key, value|
        self.class.defined_attributes.include?(key)
      }
    end

    def self.attributes(*attrs)
      attrs.each do |attr|
        defined_attributes << attr
        define_method(attr) do
          instance_variable_get("@#{attr}")
        end
      end
    end

    def self.defined_attributes
      @attributes ||= []
    end

    def self.inherited(subclass)
      subclass.send(
        :instance_variable_set,
        :@attributes,
        defined_attributes.dup
      )
    end
  end
end
