
begin
  gem 'faraday'
  require 'faraday'
rescue Gem::LoadError
  # suppress
end


module WCC::Data
  if defined?(Faraday)
    class FaradayClientAppTokenAuth < Faraday::Middleware
      class RedisCache
        DEFAULT_CACHE_KEY = "org.watermark.data.client-app-token-cache-store"
        attr_reader :connection, :cache_key

        def initialize(connection, cache_key: DEFAULT_CACHE_KEY)
          @connection = connection
          @cache_key = cache_key
        end

        def [](hostname)
          connection.call { |r| r.hget cache_key, hostname }
        end

        def []=(hostname, token)
          if token
            connection.call { |r| r.hset cache_key, hostname, token }
          else
            connection.call { |r| r.hdel cache_key, hostname }
          end
        end
      end

      attr_reader :cache, :token_factory

      def initialize(app, cache: default_cache, token_factory: method(:default_token_factory))
        super(app)
        @cache = cache
        @token_factory = token_factory
      end

      def token_for(host)
        @cache[host] ||= token_factory.(host)
      end

      def call(env, retry_on_forbidden: true)
        request_body = env[:body]
        host = env[:url].host
        env[:request_headers]["Authorization"] = "Bearer #{token_for(host)}"
        @app.call(env).on_complete do |response_env|
          response = Faraday::Response.new(response_env)
          cache[host] = nil unless response.success?
          if response.status == 403 && retry_on_forbidden
            env[:body] = request_body
            call(env, retry_on_forbidden: false)
          end
        end
      end

      private

      def default_token_factory(host)
        WCC::Data::Nucleus::ClientAppToken.create(hostname: host).token
      end

      def default_cache
        RedisCache.new(Sidekiq.method(:redis))
      end
    end
  else
    class FaradayClientAppTokenAuth
      def initialize(app, cache: default_cache, token_factory: method(:default_token_factory))
        raise NotSupportedError, 'Please include the "faraday" gem to use this authentication method'
      end
    end
  end
end

if defined?(Faraday)
  Faraday::Middleware.register_middleware :client_app_token => lambda { WCC::Data::FaradayClientAppTokenAuth }
end

