module WCC::Data::Mapper

  module Attributes

    def self.included(receiver)
      receiver.send(:include, InstanceMethods)
      receiver.extend(ClassMethods)
    end

    module InstanceMethods
      attr_reader :attributes

      def initialize(attributes={})
        @attributes = attributes
      end

      def [](attribute)
        self.class.attributes.fetch(attribute.to_s) {
          raise KeyError, "The attribute #{attribute} is not defined"
        }
        @attributes[attribute.to_s]
      end

      def []=(attribute, value)
        self.class.attributes.fetch(attribute.to_s) {
          raise KeyError, "The attribute #{attribute} is not defined"
        }
        @attributes[attribute.to_s] = value
      end

    end

    module ClassMethods

      def attribute(name, options={})
        attributes[name.to_s] = options.freeze
        define_method(name) { self[name] }
        define_method("#{name}=") { |val| self[name] = val } if options[:writer]
      end

      def attributes
        @attributes ||= {}
      end

      def inherited(subclass)
        super
        subclass.instance_variable_set(:@attributes, attributes.dup)
      end

    end

  end

end
