module WCC::Data::Nucleus

  class User < Base
    set_endpoint :nucleus, "api/v1/users/"

    attribute :id
    attribute :first_name
    attribute :last_name
    attribute :email
    attribute :access_level_id
    attribute :arena_id, writer: true
    attribute :rock_id, writer: true
    attribute :created_at
    attribute :updated_at
    attribute :user_reviewed_at, writer: true
    attribute :contact
    attribute :address

    SAVE_ATTRS = %i[arena_id rock_id user_reviewed_at]

    def contact
      @contact ||= Contact.new self[:contact].merge("user_id" => id)
    end

    def address
      @address ||= Address.new self[:address].merge("user_id" => id)
    end

    def update_attributes(attrs={})
      self.class.endpoint.update(id, user: attrs)
    end

    def save
      update_attributes saveable_attributes
    end

    def saveable_attributes
      attributes
        .select { |key, _| SAVE_ATTRS.include?(key.to_sym) }
        .merge(contact: contact.attributes)
        .merge(address: address.attributes)
    end

    def self.search(string, options={})
      list(options.merge(search: string))
    end
  end

end

