module WCC::Data
  class RackClientAppTokenAuth
    class RedisCache
      DEFAULT_CACHE_KEY = "org.watermark.wcc-api.client-app-token-cache-store"
      DEFAULT_CACHE_LENGTH = 600

      attr_reader :connection, :cache_key, :cache_length

      def initialize(connection, cache_key: DEFAULT_CACHE_KEY, cache_length: DEFAULT_CACHE_LENGTH)
        @connection = connection
        @cache_key = cache_key
        @cache_length = cache_length
      end

      def <<(token)
        connection.call do |r|
          r.set join_key(token), "1"
          r.expire join_key(token), cache_length
        end
      end

      def find(token)
        connection.call do |r|
          r.get join_key(token)
        end
      end

      private

      def join_key(suffix)
        [cache_key, suffix].join(".")
      end
    end

    attr_reader :app, :cache, :lookup_token

    def initialize(app, cache: default_cache, lookup_token: method(:default_lookup_token))
      @app = app
      @cache = cache
      @lookup_token = lookup_token
    end

    def find(token)
      cached = cache.find(token)
      return cached if cached

      val = lookup_token.(token)

      cache << token if val

      val
    end

    def call(env)
      _, token_string = env["HTTP_AUTHORIZATION"].split(/\s+/)
      if find(token_string)
        app.call(env)
      else
        [403, {}, ['{"error":"Invalid Bearer Token"}']]
      end
    end

    private

    def default_cache
      RedisCache.new(Sidekiq.method(:redis))
    end

    def default_lookup_token(token)
      WCC::Data::Nucleus::ClientAppToken.find(token)
    rescue WCC::Data::Mapper::InvalidResponse, WCC::Data::Mapper::RecordNotFound
      nil
    end

  end

end
