module WCC::Data
  class Service
    attr_reader :uri, :connection

    VERBS = Set.new(%i[get post put patch delete])

    def initialize(args={})
      @uri = URI(args.fetch(:uri) { "" })
      @connection = args.fetch(:connection) { nil }
    end

    def merge(arg=nil)
      right = WCC::Data.Service(arg)
      self.class.new(
        uri: merge_uris(uri, right.uri),
        connection: right.connection || connection
      )
    end

    VERBS.each do |verb|
      define_method(verb) do |options|
        Response.new(
          connection.run_request(
            verb,
            merge_uris(uri, options.fetch(:uri) { "" }),
            options[:body],
            options[:headers]
          ) do |request|
            request.params.update(options[:params]) if options[:params]
          end
        )
      end
    end

    private

    def merge_uris(uri1, uri2)
      URI(uri1) + URI(uri2)
    end

  end

  def self.Service(hash_or_object)
    case hash_or_object
    when Service
      hash_or_object
    when Hash
      Service.new(hash_or_object)
    when NilClass
      Service.new
    else
      raise ArgumentError
    end
  end
end
